﻿using System;
using System.Collections;
using UnityEngine;

namespace AudioMate
{
    public static class Tools
    {
        /// <summary>
        /// Removes all whitespace from a string.
        /// </summary>
        public static string TrimAll(string str) {
            var len = str.Length;
            var src = str.ToCharArray();
            var dstIdx = 0;
            for (var i = 0; i < len; i++) {
                var ch = src[i];
                switch (ch) {
                    case '\u0020': case '\u00A0': case '\u1680': case '\u2000': case '\u2001':
                    case '\u2002': case '\u2003': case '\u2004': case '\u2005': case '\u2006':
                    case '\u2007': case '\u2008': case '\u2009': case '\u200A': case '\u202F':
                    case '\u205F': case '\u3000': case '\u2028': case '\u2029': case '\u0009':
                    case '\u000A': case '\u000B': case '\u000C': case '\u000D': case '\u0085':
                        break;
                    default:
                        src[dstIdx++] = ch;
                        break;
                }
            }
            return new string(src, 0, dstIdx);
        }

        /// <summary>
        /// Empty test for StorableStringChooserJSON choices
        /// </summary>
        public static bool IsEmptyChoice(string choice)
        {
            return string.IsNullOrEmpty(choice) || choice == "None";
        }

        /// <summary>
        /// Generates a unique ID
        /// </summary>
        public static string GenerateID()
        {
            return Guid.NewGuid().ToString("N");
        }

        /// Loaded scene file
        public static string SceneSaveName
        {
            get
            {
                var idx = SuperController.singleton.currentLoadDir.IndexOf(":/", StringComparison.Ordinal);
                if (idx >= 0)
                    return SuperController.singleton.currentLoadDir.Substring(0, idx + 2);

                return string.Empty;
            }
        }

        // Experimental fade transition routines
        /* public static IEnumerator FloatTransition(Action<float> targetFloat, float startValue, float endValue, float updateStep = 0.01f, float updateEvery = 0.001f)
        {
            var currentValue = startValue;
            var add = startValue < endValue;
            while (Math.Abs(currentValue - endValue) > updateStep / 100) {
                var time = 0f;
                while (time < updateEvery)
                {
                    time += Time.deltaTime;
                    yield return null;
                }
                currentValue += add ? updateStep : -updateStep;
                targetFloat (currentValue);
            }
            targetFloat (endValue);
        }

        public static IEnumerator LerpMaterialColor(Material targetMaterial, float lerpDuration, Color startColor, Color targetColor)
        {
            var lerpStartTime = Time.time;
            var lerpItBaby = true;
            while (lerpItBaby)
            {
                yield return new WaitForEndOfFrame();
                var lerpProgress = Time.time - lerpStartTime;
                if (targetMaterial != null)
                {
                    targetMaterial.color = Color.Lerp(startColor, targetColor, lerpProgress / lerpDuration);
                }
                else
                {
                    lerpItBaby = false;
                }


                if (lerpProgress >= lerpDuration)
                {
                    lerpItBaby = false;
                }
            }
        } */
    }
}
