﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;

namespace AudioMate
{

    public abstract class UIComponent : MonoBehaviour, IUIComponent
    {
        protected AudioMateController _c;
        public bool isBindComplete;

        public virtual void Init(AudioMateController controller)
        {
            if (controller == null) return;
            _c = controller;
            _Init();
        }

        protected virtual void _Init()
        {
            // ...
        }

        public virtual void Bind()
        {
            if (_c == null || _c.collections == null) return;
            _c.collections.OnActiveCollectionSelected.AddListener(OnActiveCollectionSelected);
            _c.collections.OnActiveCollectionUpdated.AddListener(OnActiveCollectionUpdated);
            _c.collections.OnActiveCollectionNameChanged.AddListener(OnActiveCollectionNameChanged);
            _c.OnContainingAtomNameChanged.AddListener(OnContainingAtomNameChanged);
            isBindComplete = true;
            _Bind();
        }

        protected virtual void _Bind()
        {
            // ...
        }

        public virtual void OnDestroy()
        {
            // ...
        }

        public virtual void OnActiveCollectionSelected(CollectionManager.ActiveCollectionSelectedEventArgs eventArgs)
        {
            // ...
        }

        public virtual void OnActiveCollectionUpdated()
        {
            // ...
        }

        public virtual void OnActiveCollectionNameChanged(CollectionManager.ActiveCollectionNameChangedEventArgs eventArgs)
        {
            // ...
        }

        public virtual void OnContainingAtomNameChanged(AudioMateController.ContainingAtomNameChangedEventArgs eventArgs)
        {
            // ...
        }

        protected virtual void Log(string message)
        {
            if (_c == null) return;
            _c.Log($" [Component]: {message}");
        }
    }
}
